unit PackageAPI;

interface
uses Windows;

const
  APIRET_SUCCESS = 0;
  // -- Added 2011-11-21.
  APIRET_FAILURE = 1;
  APIRET_VIRTFILES_DB_ERROR = 2;
  APIRET_VIRTFILES_ZIP_ERROR = 3;
  APIRET_NOT_FOUND = 5;
  APIRET_INVALID_PARAMETER = 6;
  APIRET_FILE_CREATE_ERROR = 7;
  APIRET_PE_RESOURCE_ERROR = 8;
  APIRET_MEMORY_ERROR = 9;
  APIRET_COMMIT_ERROR = 10;
  APIRET_VIRTREG_DEPLOY_ERROR = 11;
  APIRET_OUTPUT_ERROR = 12;
  APIRET_INSUFFICIENT_BUFFER = 13;
  APIRET_LOADLIBRARY_ERROR = 14;
  APIRET_VIRTFILES_INI_ERROR = 15;
  APIRET_APP_NOT_DEPLOYED = 16;
  APIRET_INSUFFICIENT_PRIVILEGES = 17;
  APIRET_32_64_BIT_MISMATCH = 18;
  APIRET_DOTNET_REQUIRED = 19;
  APIRET_CANCELLED = 20;
  // -- end added..
  VIRT_FILE_FLAGS_ISFILE = $0001; // File or directory?
  VIRT_FILE_FLAGS_DELETED = $0002; // Deleted by virtual app (NOT_FOUND)
  VIRT_FILE_FLAGS_DEPLOYED = $0008; // Set upon first file opening
  (* Reserved in CPP and therefore commented out!
  VIRT_FILE_FLAGS_DISCONNECTED  = $0010;	// Set when on-disk file is modified from DB
  *)
  VIRT_FILE_FLAGS_PKG_FILE = $0020; // File/dir is part of the original package (as opposed to files newly-added to sandbox during package use)



  SANDBOXFLAGS_PASSTHROUGH = 1;
  SANDBOXFLAGS_COPY_ON_WRITE = 2;
  SANDBOXFLAGS_FULL_ISOLATION = 3;

  MAX_APPID_LENGTH = 128;

  PackagerDLL = 'PackagerDll.dll';
  EmptyWidestring = #0#0#0#0;

type
  PACKAGE_HANDLE = Pointer;

  // Custom type for ISOLATIONMODE...Belongs to Get/SetIsolationMode(hPkg) Helperfunctions
  TISOLATIONMODE = ( ISOLATIONMODE_CUSTOM, ISOLATIONMODE_ISOLATED, ISOLATIONMODE_FULL_ACCESS, ISOLATIONMODE_DATA );

  VIRTFS_ENUM_CALLBACK = function(
    Data : Pointer;
    FileName : PWideChar;
    FileFlags : Cardinal;
    CreationTime : LARGE_INTEGER;
    LastAccessTime : LARGE_INTEGER;
    LastWriteTime : LARGE_INTEGER;
    ChangeTime : LARGE_INTEGER;
    EndOfFile : LARGE_INTEGER;
    FileAttributes : Cardinal
  ) : LongBool; stdcall;

(* These structures are untested and may not be entirely accurate *)
  VIRT_PROCESS = record
    PID : DWORD;
    Flags : DWORD;
  End;

  PRUNNING_APP = ^RUNNING_APP;
  RUNNING_APP = record
    Version: DWORD;
    SerialID: DWORD;
    CarrierExeName: packed array[0..(Windows.MAX_PATH * 4) - 1] of WideChar;
    CarrierPID: DWORD;
    StartTickTime: DWORD;
    SyncStreamingDuration: DWORD;
    TotalPIDs: DWORD;
    PIDs: packed array[0..Pred(2048)] of VIRT_PROCESS;
    AppID: packed array[0..MAX_APPID_LENGTH - 1] of WideChar;
    FriendlyName: packed array[0..MAX_APPID_LENGTH - 1] of WideChar;
  end;

  RUNNINGAPP_ENUM_CALLBACK = function(
    Data: Pointer;
    RunningApp: PRUNNING_APP
    ): LongBool; Stdcall;

  PDEPLOYED_APP = ^DEPLOYED_APP;
  DEPLOYED_APP = record
    AppID: packed array[0..MAX_APPID_LENGTH - 1] of WideChar;
  end;

  DEPLOYEDAPP_ENUM_CALLBACK = function(
    Data: Pointer;
    AppID: PWideChar
    ): LongBool; StdCall;
(* End Untested *)


(*
  Functions for DLL below...
*)

function PackageOpen(
    PackageExeFile: PWideChar;
    Reserved: DWORD;
    var hPkg: PACKAGE_HANDLE
  ): Integer; stdcall; external PackagerDLL;

function PackageCreate(
    AppID: PWideChar;
    AppVirtDLL: PWideChar;
    LoaderExe: PWideChar;
    var hPkg: PACKAGE_HANDLE
  ): Integer; stdcall; External PackagerDLL;

function PackageSave(
    hPkg: PACKAGE_HANDLE;
    OutFileName: PWideChar
  ): Integer; stdcall; external PackagerDLL;

function PackageClose(
    hPkg: PACKAGE_HANDLE
  ): Integer; stdcall; external PackagerDLL;

function PackageGetProperty(
    hPkg: PACKAGE_HANDLE;
    Name: PWideChar;
    Value: PWideChar;
    ValueLen: DWORD
  ): Integer; stdcall; external PackagerDLL;

function PackageSetProperty(
    hPkg: PACKAGE_HANDLE;
    Name: PWideChar;
    Value: PWideChar
  ): Integer; stdcall; external PackagerDLL;

function PackageSetIconFile(
  hPkg: PACKAGE_HANDLE;
  FileName: PWideChar
  ): Integer; stdcall; external PackagerDLL;

function PackageBuildStreaming(
    hPkg: PACKAGE_HANDLE;
    StreamingServerPath: PWideChar;
    OutputDir: PWideChar
  ): Integer; stdcall; external PackagerDLL;

function VirtFsEnum(
    hPkg: PACKAGE_HANDLE;
    Callback: VIRTFS_ENUM_CALLBACK;
    Data: Pointer
  ): Integer; stdcall; external PackagerDLL;

function VirtFsAdd(
    hPkg: PACKAGE_HANDLE;
    SrcFileName: PWideChar;
    DestFileName: PWideChar;
    bVariablizeName: LongBool
  ): Integer; stdcall; external PackagerDLL;

//Most below was missing 2011-11-21
function VirtFsAddEx(
    hPkg: PACKAGE_HANDLE;
    SrcFileName: PWideChar;
    _DestFileName: PWideChar;
    bVariablizeName: LongBool;
    FileFlags: Cardinal
  ): Integer; stdCall; External PackagerDLL;

function VirtFsAddEmptyDir(
    hPkg: PACKAGE_HANDLE;
    DirName: PWideChar;
    bVariablizeName: LongBool
  ): Integer; stdCall; External PackagerDLL;

function VirtFsExtract(
    hPkg: PACKAGE_HANDLE;
    FileName: PWideChar;
    TargetDir: PWideChar
  ): Integer; stdCall; External PackagerDLL;

//Moved 2011-11-21, to correspond with location in CPP.
function VirtFsDelete(
    hPkg: PACKAGE_HANDLE;
    FileName: PWideChar
  ): Integer; stdcall; external PackagerDLL;

function VirtFsGetFileFlags(
    hPkg: PACKAGE_HANDLE;
    FileName: PWideChar;
    var FileFlags: Cardinal
  ): Integer; stdcall; external PackagerDLL;

function VirtFsSetFileFlags(
    hPkg: PACKAGE_HANDLE;
    FileName: PWideChar;
    FileFlags: Cardinal
  ): Integer; stdcall; external PackagerDLL;

//Moved 2011-11-21, to correspond with location in CPP.
function VirtRegGetWorkKey(
    hPkg: PACKAGE_HANDLE;
    WorkKey: PWideChar;
    WorkKeyLen: DWORD
  ): Integer; stdcall; external PackagerDLL;

function VirtRegGetWorkKeyEx(
    hPkg: PACKAGE_HANDLE;
    WorkKey: PWideChar;
    WorkKeyLen: DWORD;
    hAbortEvent: THandle
  ): Integer; stdcall; external PackagerDLL;

function VirtRegSaveWorkKey(
    hPkg: PACKAGE_HANDLE
  ): Integer; stdcall; external PackagerDLL;

function SandboxGetRegistryFlags(
    hPkg: PACKAGE_HANDLE;
    Path: PWideChar;
    bVariablizeName: LongBool;
    var SandBoxFlags: DWORD
  ): Integer; stdcall; external PackagerDLL;

function SandboxGetFileFlags(
    hPkg: PACKAGE_HANDLE;
    Path: PWideChar;
    bVariablizeName: LongBool;
    var SandBoxFlags: DWord
  ): Integer; stdcall; external PackagerDLL;

function SandboxSetRegistryFlags(
    hPkg: PACKAGE_HANDLE;
    Path: PWideChar;
    bVariablizeName: LongBool;
    SandboxFlags: DWord
  ): Integer; stdcall; external PackagerDLL;

function SandboxSetFileFlags(
    hPkg: PACKAGE_HANDLE;
    Path: PWideChar;
    bVariablizeName: LongBool;
    SandboxFlags: DWord
  ): Integer; stdcall; external PackagerDLL;


// But we keep them to have them as they exist in CPP.
Function QuickReadIni(
    PackageExeFile: PWideChar;
    IniStr: PWideChar;
    IniStrLen: DWord
  ) : Integer; stdcall; external PackagerDLL;

function RunningAppEnum(
    Callback: RUNNINGAPP_ENUM_CALLBACK;
    Data: Pointer
  ): Integer; stdcall; external PackagerDLL;

function DeployedAppEnum(
    Callback: RUNNINGAPP_ENUM_CALLBACK;
    Data: Pointer
  ): Integer; stdcall; external PackagerDLL;

function DeployedAppGetDir(
    AppID: PWideChar;
    BaseDirName: PWideChar;
    BaseDirNameLen: DWord
  ): Integer; StdCall; External PackagerDLL;

// Helper functions
function ApiSuccess( ApiResult: Integer; PromptOnError: Boolean = False ): Boolean;

// Converted from VB Example...
function SetProperty( hPkg: PACKAGE_HANDLE; const Name, Value: WideString ): Boolean;
function GetProperty( hPkg: PACKAGE_HANDLE; const Name: WideString ): string;

// Converted from VB Example...
function GetIsolationMode( hPkg: PACKAGE_HANDLE ): TISOLATIONMODE;
procedure SetIsolationMode( hPkg: PACKAGE_HANDLE; IsolationMode: TISOLATIONMODE );

implementation

uses SysUtils;

// == Helper Functions..

function ApiSuccess( ApiResult: Integer; PromptOnError: Boolean = False ): Boolean;
begin
  Result := ( ApiResult = APIRET_SUCCESS );
  if not Result and PromptOnError then begin
    // Changed from ShowMessage since we then don't need to include Dialogs.
    MessageBox( 0, PChar( Format( 'Failure: %d', [ApiResult] ) ), 'Error', MB_ICONERROR or MB_OK );
  end;
end;

(* Largely untested code below, but appears functional *)

function GetPWideChar( const aWideString: WideString ): PWidechar;
begin
  if Length( aWideString ) > 0 then
    Result := @aWideString[1]
  else
    Result := @EmptyWideString[1];
end;

function SetProperty( hPkg: PACKAGE_HANDLE; const Name, Value: WideString ): Boolean;
begin
  Result := PackageSetProperty( hPkg, GetPWideChar( Name ), GetPWideChar( Value ) ) = APIRET_SUCCESS;
end;

function GetProperty( hPkg: PACKAGE_HANDLE; const Name: WideString ): string;
const
  MAX_STRING_LENGTH = 64 * 1024;
var
  buf: array[1..MAX_STRING_LENGTH] of char;
begin
  Fillchar( Buf[1], SizeOf( Buf ), 0 );
  if (PackageGetProperty( hPkg, GetPWideChar( Name ), @buf[1], MAX_STRING_LENGTH - 1 ) = APIRET_SUCCESS) then begin
    if Buf[1] = #0 then
      Result := ''
    else
      Result := WideCharToString( @Buf[1] );
  end;
end;

function GetIsolationMode( hPkg: PACKAGE_HANDLE ): TISOLATIONMODE;
  function GetFileSandbox( const Path: WideString; bVariablizeName: Boolean = FALSE ): Cardinal;
  begin
    SandboxGetFileFlags( hPkg, GetPWideChar( Path ), bVariablizeNAme, Result );
  end;

  function GetRegistrySandbox( const Path: WideString; bVariablizeName: Boolean = FALSE ): Cardinal;
  begin
    SandboxGetRegistryFlags( hPkg, GetPWideChar( Path ), bVariablizeName, Result );
  end;

begin
  if ( GetFileSandbox('') = SANDBOXFLAGS_COPY_ON_WRITE ) and
    ( GetRegistrySandbox('') = SANDBOXFLAGS_COPY_ON_WRITE ) and
    ( GetFileSandbox('%Personal%') = SANDBOXFLAGS_PASSTHROUGH ) and
    ( GetFileSandbox('%Desktop%') = SANDBOXFLAGS_PASSTHROUGH ) and
    ( GetFileSandbox('UNC') = SANDBOXFLAGS_PASSTHROUGH ) then
    Result := ISOLATIONMODE_DATA
  else if ( GetFileSandbox('') = SANDBOXFLAGS_COPY_ON_WRITE ) and
    ( GetRegistrySandbox('') = SANDBOXFLAGS_COPY_ON_WRITE ) then
    Result := ISOLATIONMODE_ISOLATED
  else if ( GetFileSandbox('') = SANDBOXFLAGS_PASSTHROUGH ) and
    ( GetRegistrySandbox('') = SANDBOXFLAGS_PASSTHROUGH ) then
    Result := ISOLATIONMODE_FULL_ACCESS
  else
    Result := ISOLATIONMODE_CUSTOM;
end;

procedure SetIsolationMode( hPkg: PACKAGE_HANDLE; IsolationMode: TISOLATIONMODE );
  function SetRegistrySandbox( const Path: WideString; const SandboxFlags: DWord; bVariablizeName: Boolean = FALSE ): Boolean;
  begin
    Result := SandboxSetRegistryFlags( hPkg, GetPWideChar( Path ), bVariablizeName, SandboxFlags ) = APIRET_SUCCESS;
  end;

  function SetFileSandbox( const Path: WideString; const SandboxFlags: DWord; bVariablizeNAme: Boolean = FALSE ): Boolean;
  begin
    Result := SandboxSetFileFlags( hPkg, GetPWideChar( Path ), bVariablizeName, SandboxFlags ) = APIRET_SUCCESS;
  end;

var
  SandboxMode: Cardinal;
begin
  SandboxMode := 0;
  if (IsolationMode = ISOLATIONMODE_ISOLATED) or (IsolationMode = ISOLATIONMODE_DATA) then
    SandboxMode := SANDBOXFLAGS_COPY_ON_WRITE
  else if (IsolationMode = ISOLATIONMODE_FULL_ACCESS) then
    SandboxMode := SANDBOXFLAGS_PASSTHROUGH;

  if (SandboxMode <> 0) then begin
    SetFileSandbox('', SandboxMode);
    SetRegistrySandbox('', SandboxMode);
  end;

  if (IsolationMode = ISOLATIONMODE_DATA) then begin
    SetProperty(hPkg, 'DataMode', 'TRUE');
    SetFileSandbox('%Personal%', SANDBOXFLAGS_PASSTHROUGH);
    SetFileSandbox('%Desktop%', SANDBOXFLAGS_PASSTHROUGH);
    SetFileSandbox('UNC', SANDBOXFLAGS_PASSTHROUGH);
  end
  else if (AnsiCompareText(GetProperty(hPkg, 'DataMode'), 'TRUE') = 0) then begin
    SetProperty(hPkg, 'DataMode', 'FALSE');
    SetFileSandbox('%Personal%', SandboxMode);
    SetFileSandbox('%Desktop%', SandboxMode);
    SetFileSandbox('UNC', SandboxMode);
  end;
end;

// =======================================================================

end.